import {
  AbstractControl,
  FormControl,
  FormGroup,
  FormGroupDirective,
  NG_VALIDATORS,
  NgForm,
  ValidationErrors, Validator,
  ValidatorFn
} from '@angular/forms';
import {ErrorStateMatcher} from '@angular/material/core';
import {Directive} from '@angular/core';

export class ActivityModalValidator {
  static weekRangeValidator: (fromWeekName: string, toWeekName: string) => ValidatorFn =
    (fromWeekName: string, toWeekName: string) => (fg: AbstractControl): ValidationErrors | null => {
      const fromWeek: number = (fg as FormGroup).get(fromWeekName)?.value;
      const statusIsFinished: boolean = (fg as FormGroup).get('status')?.value === 'finished';
      const toWeek: number = (fg as FormGroup).get(toWeekName)?.value;
      const weekRangeInvalid = statusIsFinished ? fromWeek == null : (fromWeek == null || toWeek == null || fromWeek > toWeek);
      return weekRangeInvalid ? { weekRangeInvalid } : null;
    }

  static noteMissingOnFinishedActivityValidator: ValidatorFn = (control: AbstractControl): ValidationErrors | null => {
    const status = control.get('status');
    const note = control.get('note');
    if (status && status.value === 'finished') {
      if (!(note && note.value?.length > 0)) {
        return { noteIsMissing: true };
      }
    }
    return null;
  }
}

export class ActivityModalErrorStateMatcher implements ErrorStateMatcher {
  isErrorState(control: FormControl | null, form: FormGroupDirective | NgForm | null): boolean {
    return !!control?.parent?.invalid && control?.touched;
  }
}
