import { DataCardWrapper } from './dataCard.wrapper';
import { DataCard } from '@nspop/gm-web-facade-api';

describe('DataCardWrapper', () => {
  it('reads and writes values for all properties', () => {
    const dataCard = {} as unknown as DataCard;
    const subject = new DataCardWrapper(dataCard);
    const yesNoEnumMappedProperties = [ 'obstetricAssessmentRequired', 'socialServicesRequired' ];
    let length = 2;
    Object.getOwnPropertyNames(DataCardWrapper.prototype)
      .filter(prop => {
        return typeof (subject as any)[prop] === 'undefined'
          && !prop.includes('Section') // Don't write sections to strings to avoid '15 value'.property access
          && !prop.toLocaleLowerCase().includes('address'); // Same with address
      })
      .forEach((property, index) => {
        const valueSet = `${index} value`;
        (subject as any)[property] = valueSet;
        const stringified = JSON.stringify(dataCard);
        if ((subject as any)[property]) {
          // See DGAF-2216 - we are doing a special mapping for some properties to convert them into boolean instead of YesNoEnum due to backend limitations
          if (yesNoEnumMappedProperties.includes(property)) {
            (subject as any)[property] = undefined;
            expect((subject as any)[property]).toBe(undefined, `failed to read back the undefined in '${property}' (Special YesNoEnum mapped to boolean)`);

            (subject as any)[property] = true;
            expect((subject as any)[property]).toBe(true, `failed to read back true set in '${property}' (Special YesNoEnum mapped to boolean)`);

            (subject as any)[property] = false;
            expect((subject as any)[property]).toBe(false, `failed to read back false set in '${property}' (Special YesNoEnum mapped to boolean)`);
          } else {
            expect((subject as any)[property]).toBe(valueSet, `failed to read back the value set in '${property}'`);
            expect(stringified.length > length)
              .toBeTruthy(`Expected length of dataCard as json to be greater than ${length} after setting ${property} but it was ${stringified.length}`);
          }
        }
        length = stringified.length;
      });
    // Randomly selected tests
    expect(dataCard.prenatalRiskAssessmentSection?.riskAssessment2?.multiplicityRiskAssessment?.comment).toContain(' value');
    expect(dataCard.alcoholDrinkingBehaviorSection?.weeklyConsumptionBeforePregnancy?.consumption?.value).toContain(' value');
  });
});
