import { AllowOnlyNumbersDirective } from './allow-only-numbers.directive';
import {AbstractControl} from '@angular/forms';

describe('AllowOnlyNumbersDirective', () => {
  it('should create an instance', () => {
    const directive = new AllowOnlyNumbersDirective();
    expect(directive).toBeTruthy();
  });

  it('should not allow trailing dots', () => {
    const directive = new AllowOnlyNumbersDirective();
    const result = directive.validate({value: '123.'} as AbstractControl);
    expect(result?.appOnlyNumbersDirective).toBe('No ending decimal separators!');
  });
  it('should validate when value is null', () => {
    const directive = new AllowOnlyNumbersDirective();
    const result = directive.validate({value: null} as AbstractControl);
    expect(result).toBeNull();
  });
  it('should not validate when leading with zeroes', () => {
    const directive = new AllowOnlyNumbersDirective();
    const result = directive.validate({value: '01'} as AbstractControl);
    expect(result).toEqual({ appOnlyNumbersDirective: 'No leading zeros!' });
  });
  it('should not validate when ending with decimals', () => {
    const directive = new AllowOnlyNumbersDirective();
    const result = directive.validate({value: '123.'} as AbstractControl);
    expect(result).toEqual({ appOnlyNumbersDirective: 'No ending decimal separators!' });
  });
  it('should not validate when not a number', () => {
    const directive = new AllowOnlyNumbersDirective();
    const result = directive.validate({value: 'abcdef'} as AbstractControl);
    expect(result).toEqual({ appOnlyNumbersDirective: 'Value is not a valid or accepted number!' });
  });
});
