import { Injectable } from '@angular/core';
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { KeepAliveConfigService } from '../app/services/keep-alive-config.service';
import { debounce, DebouncedFunc } from 'lodash-es';
import { environment } from '../environments/environment';

export interface KeepAliveContext {
  configuredIdleTimeout: number;
}

@Injectable()
export class RefreshInterceptor implements HttpInterceptor {
  private debouncedForcedLogoutFn: DebouncedFunc<() => void> | undefined;
  private observable: Observable<number> | undefined;

  constructor(private keepAliveConfigService: KeepAliveConfigService) {
  }

  intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>> {
    if (!this.observable) {
      this.observable = this.keepAliveConfigService.keepAliveContext
        .pipe(
          take(1),
          map((keepAliveConfig: KeepAliveContext) => keepAliveConfig.configuredIdleTimeout * 1000)
        );
      this.observable.subscribe((configuredIdleTimeout) => {
        this.debouncedForcedLogoutFn = debounce(() => {
          window.location.assign(environment.base_path + '/saml/Logout');
        }, configuredIdleTimeout);
      });
    }
    if (this.debouncedForcedLogoutFn) {
      this.debouncedForcedLogoutFn();
    }
    return next.handle(request);
  }
}
